define(['underscore', 'mundo/exec'], function(_, Exec) {
    console.info("GPS Load!");

    function GPS(services) {
        var _q = services.$q;
        var _http = services.$http;

        var _uri = "http://api.map.baidu.com/geocoder/v2/";
        var _ak = "04fa949bd0fa93a3b651d4b24679c2a8";
        var _output = "json";
        var _coordType = "wgs84ll";

        var _apiUri = _uri + '?ak=' + _ak + '&output=' + _output + '&coordType=' + _coordType;

        function formatTime(millisec, format) {
            var date = new Date();
            date.setTime(millisec);
            var yyyy = date.getFullYear();
            var MM = (date.getMonth() + 101 + '').slice(1);
            var dd = (date.getDate() + 100 + '').slice(1);
            if (format === 'yyyy-MM-dd') {
                return yyyy + '-' + MM + '-' + dd;
            } else if (format === 'yyyy-MM-dd hh:mm:ss') {
                var hh = (date.getHours() + 100 + '').slice(1);
                var mm = (date.getMinutes() + 100 + '').slice(1);
                var ss = (date.getSeconds() + 100 + '').slice(1);
                return yyyy + '-' + MM + '-' + dd + ' ' + hh + ':' + mm + ':' + ss;
            }
        }

        // Mundo只返回gps
        function getGps() {
            var deferred = _q.defer();

            Exec(function(args) {
                    var result = {
                        latitude: args.coords.latitude,
                        longitude: args.coords.longitude,
                        timestamp: formatTime(args.timestamp, 'yyyy-MM-dd hh:mm:ss')
                    };

                    deferred.resolve(result);
                },
                function(args) {
                    var errorMsg = "";
                    switch (args.code.toString()) {
                        case "1":
                            // errorMsg = "不允许定位"; // PERMISSION DENIED
                            errorMsg = "不允许定位，请打开手机的GPS开关";
                            break;
                        case "2":
                            errorMsg = "当前位置不可用"; // POSITION UNAVAILABLE
                            break;
                        case "3":
                            errorMsg = "已超时"; // TIMEOUT
                            break;
                        default:
                            errorMsg = "未知错误";
                    }
                    deferred.reject(errorMsg);
                }, "Location", "getCurrentPosition", [{ coordType: 1, reverseGeocoding: false, enableHighAccuracy: false, timeout: 20000, type: 1 }]);

            return deferred.promise;

        }

        function getLocation() {
            var deferred = _q.defer();

            // enableHighAccuracy = false, 精度100米
            Exec(function(args) {
                    var result = {
                        latitude: args.coords.latitude,
                        longitude: args.coords.longitude,
                        timestamp: formatTime(args.timestamp, 'yyyy-MM-dd hh:mm:ss'),
                        address: args.address
                    };

                    deferred.resolve(result);
                },
                function(args) {
                    var errorMsg = "";
                    switch (args.code.toString()) {
                        case "1":
                            // errorMsg = "不允许定位"; // PERMISSION DENIED
                            errorMsg = "不允许定位，请打开手机的GPS开关";
                            break;
                        case "2":
                            errorMsg = "当前位置不可用"; // POSITION UNAVAILABLE
                            break;
                        case "3":
                            errorMsg = "已超时"; // TIMEOUT
                            break;
                        default:
                            errorMsg = "未知错误";
                    }
                    deferred.reject(errorMsg);
                }, "Location", "getCurrentPosition", [{ coordType: 1, reverseGeocoding: true, enableHighAccuracy: false, timeout: 20000, type: 1 }]);

            return deferred.promise;
        }

        function reverseGeocode(latitude, longitude) {
            var defer = _q.defer();
            _http.get(_apiUri + "&location=" + latitude + ',' + longitude)
                .success(function(data) {
                    console.log(angular.toJson(data), 'reverseGeocode');
                    var address = '';
                    if (data.status == 0) {
                        address = data.result && data.result.formatted_address;
                    }
                    defer.resolve(address);
                })
                .error(function(data) {
                    defer.reject("百度API错误：" + data.message);
                });

            return defer.promise;
        }

        function getLocation2() {
            var defer = _q.defer();
            getGps().then(function(location) {
                var latitude = location.latitude, longitude = location.longitude;
                reverseGeocode(latitude, longitude)
                    .then(function(address) {
                        location.address = address;
                        defer.resolve(location);
                    }, function(baiduError) {
                        defer.reject(baiduError);
                    });
            }, function(error) {
                defer.reject(error);
            });

            return defer.promise;
        }

        return {
            getLocation: getLocation,
            getLocation2: getLocation2,
            getGps: getGps,
            reverseGeocode: reverseGeocode
        }
    }

    return GPS;
});
