define(['underscore', 'angular', 'core/page', 'native/alert', 'util/uuid'], function(_, angular, Page, Alert, UUID) {
    console.log("Task Engine Load!");

    function TaskEngine(services, task) {
        this.$location = services.$location;
        this.$q = services.$q;
        this.$contextUtils = services.$contextUtils;
        this.$stepFactory = services.$stepFactory;

        this.task = task;

        /* 当前页：title, logo, range 等 */
        this.curPage = undefined;

        /* 当前页在 this.pages 中的索引 */
        this.pagePos = 0;

        this.pageHistory = [];

        this.answers = {};
    }

    TaskEngine.prototype.start = function() {
        console.log("-- Task Engine Start!!! --");
        var p = this.task.pages[this.pagePos++];
        this.curPage = new Page(this.pagePos, p, this.task.steps, this.$stepFactory);
    };

    TaskEngine.prototype.nextPage = function() {
        var tips = this.checkAnswers();
        if (tips != "") {
            Alert(this.$q, window.wdBridge, tips);
            return false;
        }
        var p = this.task.pages[this.pagePos++];
        this.curPage = new Page(this.pagePos, p, this.task.steps, this.$stepFactory);
        // this.$location.url('next/' + UUID.v1());
        this.$location.url('next/' + this.curPage.pageNo);
    };

    TaskEngine.prototype.prevPage = function() {
        this.pagePos--;
        var p = this.task.pages[this.pagePos-1];
        this.curPage = new Page(this.pagePos, p, this.task.steps, this.$stepFactory);
        // this.$location.url('prev/' + UUID.v1());
        this.$location.url('prev/' + this.curPage.pageNo);
    };

    TaskEngine.prototype.checkAnswers = function() {
        if (this.task.needCheck) {
            return this.curPage.checkAnswers();
        }
        return "";
    };

    TaskEngine.prototype.submit = function(alert, q) {
        console.log("-- Submit Answers !!! --");
        console.log(this.task.steps);
        console.log(this.answers);
        this.$contextUtils.submit(this.task.steps, this.answers, alert, q);
    };

    TaskEngine.prototype.stop = function() {
        console.log("-- Task Engine Stop!!! --");
    };

    TaskEngine.prototype.abort = function() {

    };

    return TaskEngine;
});