define(['angular', 'util/swipe', 'core/validator', 'core/wd.bridge', 'native/alert', 'native/get.location', 'native/choose.image', 'native/scan.code'],
    function(angular, Swipe, Validator, wdBridge, Alert, GetLocation, ChooseImage, ScanCode) {
    console.log("Warden Directives Load!");

    var wdDirectives = angular.module('wdDirectives', ['wdBridge']);

    /* 单选题 */
    wdDirectives.directive('wardenRadio', function() {
        return {
            scope: {
                step: "=step"
            },
            controller: function($scope) {
                $scope.chooseOption = function($event) {
                    var namedNodeMap = $event.target.attributes;
                    $scope.step.answer = parseInt(namedNodeMap.getNamedItem("data-option-index").value);
                    var seq = $scope.step.seq;
                    warden.answers[seq] = $scope.step.answer;
                }
            },
            templateUrl: "tpls/step/radio.html"
        }
    });

    /* 多选题 */
    wdDirectives.directive('wardenCheckbox', function() {
        return {
            scope: {
                step: "=step"
            },
            controller: function($scope) {
                $scope.chooseOption = function($event) {
                    var namedNodeMap = $event.target.attributes;
                    var optionIndex = parseInt(namedNodeMap.getNamedItem("data-option-index").value);
                    console.log($scope.step.answer);
                    console.log(optionIndex);
                    if (_.indexOf($scope.step.answer, optionIndex) == -1) {
                        if ($scope.step.exclusives) {
                            if (_.indexOf($scope.step.exclusives, optionIndex) == -1) {
                                $scope.step.answer = _.filter($scope.step.answer, function(option) {
                                    return _.indexOf($scope.step.exclusives, option) == -1;
                                });
                                $scope.step.answer.push(optionIndex);
                                $scope.step.answer = $scope.step.answer.reverse();
                            } else {
                                $scope.step.answer = [optionIndex];
                            }
                        } else { // 无排他选项设定
                            $scope.step.answer.push(optionIndex);
                            $scope.step.answer = $scope.step.answer.reverse();
                        }
                    } else {
                        $scope.step.answer = _.filter($scope.step.answer, function(option) {
                            return option != optionIndex;
                        });
                    }
                    var seq = $scope.step.seq;
                    warden.answers[seq] = $scope.step.answer;
                }
            },
            templateUrl: "tpls/step/checkbox.html"
        }
    });

    /* 填空题 */
    wdDirectives.directive('wardenInput', function() {
        return {
            scope: {
                step: "=step",
                tips: "=tips"
            },
            controller: function($scope) {
                $scope.step.answer = "";
                $scope.tips = "";
                $scope.checkAnswer = function() {
                    var step = $scope.step,
                        answer = $scope.step.answer;

                    if (step.allowEmpty && !answer) {
                        $scope.tips = "";
                    } else {
                        if (!this.allowEmpty && !answer) {
                            $scope.tips = "不允许为空";
                        } else {
                            var validator = new Validator(step.pattern, step.tips);
                            $scope.tips = validator.validate(answer);
                        }
                    }
                }
            },
            templateUrl: "tpls/step/input.html"
        };
    });

    /* 描述 */
    wdDirectives.directive('wardenDescription', function() {
        return {
            scope: {
                step: "=step"
            },
            templateUrl: "tpls/step/description.html"
        };
    });

    /* 定位 */
    wdDirectives.directive('wardenLocation', ['$q', function($q) {
        return {
            scope: {
                step: "=step"
            },
            controller: function($scope) {
                if ($scope.step.answer.latitude) {
                    $scope.locationInfo = "经度：" + $scope.step.answer.longitude + ", 纬度：" + $scope.step.answer.latitude
                        + ($scope.step.answer.address ? (", 位置：" + $scope.step.answer.address) : "");
                }

                $scope.getLocation = function() {
                    console.log("Invoke Native Method: getLocation");
                    console.log(window.wdBridge);
                    GetLocation($q, window.wdBridge).then(function(location) {
                        $scope.step.answer.longitude = location.longitude;
                        $scope.step.answer.latitude = location.latitude;
                        if (location.address) $scope.step.answer.address = location.address;
                        if (location.timestamp) $scope.step.answer.timestamp = location.timestamp;
                        console.log("get location success!");
                        $scope.locationInfo = "经度：" + location.longitude + ", 纬度：" + location.latitude
                            + (location.address ? (", 位置：" + location.address) : "");
                    }, function(error) {
                        console.log(error);
                        Alert($q, window.wdBridge, error);
                    });
                }
            },
            link: function(scope, element) {
                scope.$watch('step.answer', function(newValue, oldValue) {
                    var seq = scope.step.seq;
                    window.warden.answers[seq] = newValue;
                });
            },
            templateUrl: "tpls/step/location.html"
        }
    }]);

    /* 拍照 */
    wdDirectives.directive('wardenPhoto', ['$q', '$timeout', function($q, $timeout) {
        return {
            scope: {
                step: "=step"
            },
            controller: function($scope) {
                /* 拍照或者选择照片 */
                $scope.chooseImage = function() {
                    console.log("Invoke Native Method: chooseImage");
                    ChooseImage($q, window.wdBridge, $scope.step).then(function(images) {
                        _.each(images, function(image) {
                            image.checked = false;
                        });
                        $scope.step.answer = $scope.step.answer.concat(images);
                    }, function(error) {
                        console.log(error);
                    });
                };

                $scope.deleteImage = function() {
                    var photos = $scope.step.answer;
                    $scope.step.answer = _.filter(photos, function(photo) {
                        return !photo.checked;
                    })
                };

                $scope.clickImage = function($event) {
                    var namedNodeMap = $event.target.attributes;
                    var photoIndex = parseInt(namedNodeMap.getNamedItem("data-photo-index").value);
                    $scope.step.answer[photoIndex].checked = !$scope.step.answer[photoIndex].checked;
                };
            },
            link: function(scope, element) {
                scope.$watch('step.answer', function(newValue, oldValue) {
                    // alert(angular.toJson(newValue));
                    $timeout(function() {
                        _.each(window.document.getElementsByClassName("wx-img"), function(img) {
                            var namedNodeMap = img.attributes;
                            img.src = namedNodeMap.getNamedItem("ng-src").value;
                        });
                    });

                    var photos = _.map(newValue, function(photo) {
                        delete photo['$$hashKey'];
                        delete photo['checked'];
                        return photo;
                    });
                    var seq = scope.step.seq;
                    window.warden.answers[seq] = photos;
                });
            },
            templateUrl: "tpls/step/photo.html"
        }
    }]);

    /* 扫码 */
    wdDirectives.directive('wardenCode', ['$q', function($q) {
        return {
            scope: {
                step: "=step"
            },
            controller: function($scope) {
                $scope.scanCode = function() {
                    console.log("Invoke Native Method: scanCode");
                    ScanCode($q, window.wdBridge, $scope.step).then(function(result) {
                        $scope.step.answer = result;
                    }, function(error) {
                        console.log(error);
                    });
                }
            },
            templateUrl: "tpls/step/code.html"
        }
    }]);

    /* SKU Audit - 商品稽查 */
    wdDirectives.directive('wardenSkuAudit', ['$q', '$timeout', function($q, $timeout) {
        return {
            scope: {
                step: "=step"
            },
            controller: function($scope) {

                /*
                 *  这是一段非常坑爹的代码 凸(-_-)凸
                 *  用于在取消提交差事后，把全局answers中的键值为时间戳的伪photo答案复制回对应的SkuAudit题答案中去
                 *  todo: 如果加入循环逻辑后，该段代码可能需要重写。
                 * */
                // console.log("SkuAuditAnswer.photo.type: " + typeof $scope.step.answer.photo);
                if (typeof $scope.step.answer.photo == 'number') {
                    var key = $scope.step.answer.photo;
                    $scope.step.answer.photo = window.warden.answers[$scope.step.answer.photo];
                    delete window.warden.answers[key];
                }

                /* 确认是否存在该SKU */
                $scope.confirmSku = function($event) {
                    // todo: 微信浏览器中不支持 dataset 属性
                    // $scope.step.answer.skuFlag = parseInt($event.delegateTarget.dataset['optionIndex']);
                    var namedNodeMap = $event.target.attributes;
                    var skuFlag = parseInt(namedNodeMap.getNamedItem("data-option-index").value);
                    if (skuFlag == 2) { // 没有找到商品
                        $scope.step.answer.photo = [];
                        $scope.step.answer.code = undefined;
                    }
                    $scope.step.answer.skuFlag = skuFlag;
                };

                /* 拍照或者选择照片 */
                $scope.chooseImage = function() {
                    console.log("Invoke Native Method: chooseImage");
                    ChooseImage($q, window.wdBridge, $scope.step.photoSkuStep).then(function(images) {
                        _.each(images, function(image) {
                            image.checked = false;
                        });
                        $scope.step.answer.photo = $scope.step.answer.photo.concat(images);
                    }, function(error) {
                        console.log(error);
                    });
                };

                $scope.deleteImage = function() {
                    var photos = $scope.step.answer.photo;
                    $scope.step.answer.photo = _.filter(photos, function(photo) {
                        return !photo.checked;
                    });
                };

                $scope.clickImage = function($event) {
                    var namedNodeMap = $event.target.attributes;
                    var photoIndex = parseInt(namedNodeMap.getNamedItem("data-photo-index").value);
                    $scope.step.answer.photo[photoIndex].checked = !$scope.step.answer.photo[photoIndex].checked;
                };

                /* 扫码 */
                $scope.scanCode = function() {
                    console.log("Invoke Native Method: scanCode");
                    ScanCode($q, window.wdBridge, $scope.step).then(function(result) {
                        console.log(angular.toJson(result));
                        var matched = false;
                        // 检查是否匹配barcode
                        if ($scope.step.skuBase.barcode) {
                            _.each($scope.step.skuBase.barcode, function(barcode) {
                                if (matched) return;
                                console.log("barcode:" + barcode);
                                console.log("code:" + result.code);
                                var re = new RegExp(barcode);
                                console.log(re.test(result.code));
                                matched = re.test(result.code);
                            });
                        }
                        // 检查是否匹配二维码
                        if ($scope.step.qrcode) {
                            if ($scope.step.skuBase.qrcode == result.code) matched = true;
                        }
                        if (matched) {
                            $scope.step.answer.code = result;
                        } else {
                            Alert($q, window.wdBridge, "条码错误，请选择正确的商品");
                        }
                    }, function(error) {
                        console.log(error);
                    });
                };
            },
            link: function(scope, element) {
                /* init Swipe */
                $timeout(function() {
                    var elem = document.getElementById('mySwipe');
                    // console.log(elem);
                    window.mySwipe = new Swipe(elem, {
                        callback: function(index) {
                            var circle;
                            for(var i=0; i<scope.step.slides.length; i++) {
                                circle = angular.element(document.getElementById("slideCircle" + i));
                                if (i == index) {
                                    circle.removeClass('fa-circle-thin'); circle.addClass('fa-circle');
                                } else {
                                    circle.removeClass('fa-circle'); circle.addClass('fa-circle-thin');
                                }
                            }
                        }
                    });
                });

                scope.$watch('step.answer.photo', function(newValue, oldValue) {
                    // alert(angular.toJson(newValue));
                    $timeout(function() {
                        _.each(window.document.getElementsByClassName("wx-img"), function(img) {
                            var namedNodeMap = img.attributes;
                            img.src = namedNodeMap.getNamedItem("ng-src").value;
                        });
                    });
                });

                scope.$watch('step.answer', function(newValue, oldValue) {
                    var seq = scope.step.seq;
                    window.warden.answers[seq] = newValue;
                });
            },
            templateUrl: "tpls/step/sku_audit.html"
        }
    }]);

    return wdDirectives;
});