define(['angular', 'core/task.loader', 'core/task.engine', 'native/alert', 'core/wd.services', 'core/wd.directives'], function(angular, TaskLoader, TaskEngine, Alert) {
    console.log("Warden Load!");

    console.log("+--------------------------+");
    console.log("|    Welcome to Warden!    |");
    console.log("+--------------------------+");
    console.log("AppContext = " + window.appContext);

    var wardenApp = angular.module('wardenApp', [
        'ngRoute',
        'ngTouch',
        'ngSanitize',
        'ngAnimate',
        'wdServices',
        'wdDirectives',
	'ngIOS9UIWebViewPatch'
    ]);

    wardenApp.config(['$routeProvider', '$locationProvider', function($routeProvider, $locationProvider) {
        console.log("Warden App Config!");

        $routeProvider.
            when('/', {
                templateUrl: 'tpls/hello.html'
            }).
            when('/404', {
                templateUrl: 'tpls/404.html'
            }).

            /* Do Task */
            when('/task/:dataSource', {
                templateUrl: 'tpls/page.html'
            }).
            when('/next/:pageNo', {
                templateUrl: 'tpls/page.html'
            }).
            when('/prev/:pageNo', {
                templateUrl: 'tpls/page.html'
            }).

            when('/error/:errorCode', {
                templateUrl: 'tpls/error.html'
            }).

            /* Step Directives Testing */
            when('/step/:stepType', {
                templateUrl: 'tpls/step.html'
            }).

            otherwise({
                redirectTo: '/404'
            });

        // $locationProvider.html5Mode(true);
    }]);

    wardenApp.run(['$window', '$contextUtils', '$q', function($window, $contextUtils, $q) {
        var document = $window.document;
        var wdBridge = $window.wdBridge;

        if (window.appContext == 'weixin') {
            // todo
        } else if (window.appContext == 'wcs') {
            console.log("appContext is wcs, add event listeners for window.document.");
//            document.addEventListener('submit', function(event) {
//                $contextUtils.submit($window.warden.task.steps, $window.warden.answers);
//            });

            document.addEventListener('back', function(event) {
                event.preventDefault();
                console.log("Invoke Back Event!");
                wdBridge(function(args){
                    if (args.buttonIndex < 3) {
                        if (args.buttonIndex == 1) {
                            console.log("-- Answer to Save --");
                            console.log($window.warden.answers);
                            $contextUtils.save($window.warden.answers);
                        }
                        $contextUtils.back();
                    }
                }, function(args) {

                }, "Dialogs", "confirm", [{ message: "是否保存答案？", title: "提示", buttonLabels: ["保存", "不保存", "取消"] }]);
            });
        }
    }]);

    /* Page Controller */
    wardenApp.controller('WardenCtrl', ['$scope', '$rootScope', '$routeParams', '$location', '$http', '$q', '$stepFactory', '$pageUtils', '$contextUtils',
        function($scope, $rootScope, $routeParams, $location, $http, $q, $stepFactory, $pageUtils, $contextUtils) {
        var dataSource = $routeParams.dataSource;
        window.dataSource = dataSource;

        function linkScope() {
            $scope.next = function() {
                console.log("next page!");
                $rootScope.taskEngine.nextPage();
            };

            $scope.prev = function() {
                console.log("prev page!");
                $rootScope.taskEngine.prevPage();
            };

            $scope.submitAnswers = function() {
                console.log("submit answers!");
                $rootScope.taskEngine.submit(Alert, $q);
            };
        }

        if ($rootScope.taskEngine) { // Task Engine Exists
            linkScope();
        } else {
            if (window.assignmentId == -1 || window.assignmentId == 0) {
                Alert($q, window.wdBridge, "参数assignmentId缺失");
                $contextUtils.back();
                return false;
            }
            if (!dataSource) {
                $location.url("/error/10001");
                return false;
            }
            TaskLoader(dataSource, $http, $q, $stepFactory, $pageUtils, $contextUtils).then(function(task) {
                console.log("-- Task Created! --");
                var services = { $location: $location, $q: $q, $contextUtils: $contextUtils, $stepFactory: $stepFactory };
                $rootScope.taskEngine = new TaskEngine(services, task);
                window.warden = $rootScope.taskEngine;

//                if (window.appContext == 'web') {
//                    $rootScope.taskEngine.start();
//                    $rootScope.loaded = true;
//                    linkScope();
//                    return;
//                }

                // 加载答案
                $contextUtils.load($q).then(function(answers) {
                    if (answers && answers.answer) {
                        window.warden.answers = answers.answer;
                    } else if (answers) {
                        window.warden.answers = answers;
                    }

                    $rootScope.taskEngine.start();
                    $rootScope.loaded = true;
                    linkScope();
                }, function(errorCode) {
                    // errorCode = 10002, 加载答案失败
                    console.log(errorCode);
                    $rootScope.loaded = true;
                    $location.url("/error/" + errorCode);
                });

            }, function(errorCode) {
                console.log(errorCode);
                $rootScope.loaded = true;
                $location.url("/error/" + errorCode);
            });
        }
    }]);

    wardenApp.controller('ErrorCtrl', ['$scope', '$routeParams', function($scope, $routeParams) {
        $scope.errorCode = $routeParams.errorCode;
    }]);
});
